@@ -3,13 +3,15 @@ from __future__ import print_function
 import os
 import time
 start = time.time()
+import six
 from textwrap import fill, dedent
 from sys import stdout, stderr
 from . import public_relay
 from .. import __version__
 from ..timing import DebugTiming
 from ..errors import (WrongPasswordError, WelcomeError, KeyFormatError,
-                      TransferError)
+                      TransferError, NoTorError, UnsendableFileError,
+                      ServerConnectionError)
 from twisted.internet.defer import inlineCallbacks, maybeDeferred
 from twisted.python.failure import Failure
 from twisted.internet.task import react
@@ -104,30 +106,36 @@ def _dispatch_command(reactor, cfg, command):
 
     try:
         yield maybeDeferred(command)
-    except (WrongPasswordError, KeyFormatError) as e:
+    except (WrongPasswordError, NoTorError) as e:
         msg = fill("ERROR: " + dedent(e.__doc__))
-        print(msg, file=stderr)
+        print(msg, file=cfg.stderr)
         raise SystemExit(1)
-    except WelcomeError as e:
+    except (WelcomeError, UnsendableFileError, KeyFormatError) as e:
         msg = fill("ERROR: " + dedent(e.__doc__))
-        print(msg, file=stderr)
-        print(file=stderr)
-        print(str(e), file=stderr)
+        print(msg, file=cfg.stderr)
+        print(six.u(""), file=cfg.stderr)
+        print(six.text_type(e), file=cfg.stderr)
         raise SystemExit(1)
     except TransferError as e:
-        print("TransferError: %s" % str(e), file=stderr)
+        print(u"TransferError: %s" % six.text_type(e), file=cfg.stderr)
+        raise SystemExit(1)
+    except ServerConnectionError as e:
+        msg = fill("ERROR: " + dedent(e.__doc__)) + "\n"
+        msg += "(relay URL was %s)\n" % e.url
+        msg += six.text_type(e)
+        print(msg, file=cfg.stderr)
         raise SystemExit(1)
     except Exception as e:
         # this prints a proper traceback, whereas
         # traceback.print_exc() just prints a TB to the "yield"
         # line above ...
-        Failure().printTraceback(file=stderr)
-        print("ERROR:", e, file=stderr)
+        Failure().printTraceback(file=cfg.stderr)
+        print(u"ERROR:", six.text_type(e), file=cfg.stderr)
         raise SystemExit(1)
 
     cfg.timing.add("exit")
     if cfg.dump_timing:
-        cfg.timing.write(cfg.dump_timing, stderr)
+        cfg.timing.write(cfg.dump_timing, cfg.stderr)
 
 
 CommonArgs = _compose(
@@ -146,14 +154,29 @@ CommonArgs = _compose(
     click.option("--listen/--no-listen", default=True,
                  help="(debug) don't open a listening socket for Transit",
                  ),
+)
+
+TorArgs = _compose(
     click.option("--tor", is_flag=True, default=False,
                  help="use Tor when connecting",
                  ),
+    click.option("--launch-tor", is_flag=True, default=False,
+                 help="launch Tor, rather than use existing control/socks port",
+                 ),
+    click.option("--tor-control-port", default=None, metavar="ENDPOINT",
+                 help="endpoint descriptor for Tor control port",
+                 ),
 )
 
+@wormhole.command()
+@click.pass_context
+def help(context, **kwargs):
+    print(context.find_root().get_help())
+
 # wormhole send (or "wormhole tx")
 @wormhole.command()
 @CommonArgs
+@TorArgs
 @click.option(
     "--code", metavar="CODE",
     help="human-generated code phrase",
@@ -162,7 +185,11 @@ CommonArgs = _compose(
     "--text", default=None, metavar="MESSAGE",
     help="text message to send, instead of a file. Use '-' to read from stdin.",
 )
-@click.argument("what", required=False)
+@click.option(
+    "--ignore-unsendable-files", default=False, is_flag=True,
+    help="Don't raise an error if a file can't be read."
+)
+@click.argument("what", required=False, type=click.Path(path_type=type(u"")))
 @click.pass_obj
 def send(cfg, **kwargs):
     """Send a text message, file, or directory"""
@@ -183,6 +210,7 @@ def go(f, cfg):
 # wormhole receive (or "wormhole rx")
 @wormhole.command()
 @CommonArgs
+@TorArgs
 @click.option(
     "--only-text", "-t", is_flag=True,
     help="refuse file transfers, only accept text transfers",
@@ -245,11 +273,14 @@ def ssh():
     metavar="USER",
     help="Add to USER's ~/.ssh/authorized_keys",
 )
+@TorArgs
 @click.pass_context
-def ssh_invite(ctx, code_length, user):
+def ssh_invite(ctx, code_length, user, **kwargs):
     """
     Add a public-key to a ~/.ssh/authorized_keys file
     """
+    for name, value in kwargs.items():
+        setattr(ctx.obj, name, value)
     from . import cmd_ssh
     ctx.obj.code_length = code_length
     ctx.obj.ssh_user = user
@@ -269,8 +300,9 @@ def ssh_invite(ctx, code_length, user):
     "--yes", "-y", is_flag=True,
     help="Skip confirmation prompt to send key",
 )
+@TorArgs
 @click.pass_obj
-def ssh_accept(cfg, code, key_file, yes):
+def ssh_accept(cfg, code, key_file, yes, **kwargs):
     """
     Send your SSH public-key
 
@@ -278,6 +310,8 @@ def ssh_accept(cfg, code, key_file, yes):
     you specify (if there's only one in ~/.ssh/* that will be sent).
     """
 
+    for name, value in kwargs.items():
+        setattr(cfg, name, value)
     from . import cmd_ssh
     kind, keyid, pubkey = cmd_ssh.find_public_key(key_file)
     print("Sending public key type='{}' keyid='{}'".format(kind, keyid))
