@@ -38,6 +38,17 @@ from ..util import dict_to_bytes, bytes_to_dict
 # has claimed the nameplate has also released it, the nameplate is
 # deallocated (but they will probably keep the underlying mailbox open).
 
+# "claim" and "release" may only be called once per connection, however calls
+# across connections (assuming a consistent "side") are idempotent. [connect,
+# claim, disconnect, connect, claim] is legal, but not useful, as is a
+# "release" for a nameplate that nobody is currently claiming.
+
+# "open" and "close" may only be called once per connection. They are
+# basically idempotent, however "open" doubles as a subscribe action. So
+# [connect, open, disconnect, connect, open] is legal *and* useful (without
+# the second "open", the second connection would not be subscribed to hear
+# about new messages).
+
 # Inbound (client to server) commands are marked as "->" below. Unrecognized
 # inbound keys will be ignored. Outbound (server to client) responses use
 # "<-". There is no guaranteed correlation between requests and responses. In
@@ -220,8 +231,11 @@ class WebSocketRendezvous(websocket.WebSocketServerProtocol):
         mailbox_id = msg["mailbox"]
         assert isinstance(mailbox_id, type(""))
         self._mailbox_id = mailbox_id
-        self._mailbox = self._app.open_mailbox(mailbox_id, self._side,
-                                               server_rx)
+        try:
+            self._mailbox = self._app.open_mailbox(mailbox_id, self._side,
+                                                   server_rx)
+        except CrowdedError:
+            raise Error("crowded")
         def _send(sm):
             self.send("message", side=sm.side, phase=sm.phase,
                       body=sm.body, server_rx=sm.server_rx, id=sm.msg_id)
@@ -257,8 +271,11 @@ class WebSocketRendezvous(websocket.WebSocketServerProtocol):
                 raise Error("close without mailbox must follow open")
             mailbox_id = self._mailbox_id
         if not self._mailbox:
-            self._mailbox = self._app.open_mailbox(mailbox_id, self._side,
-                                                   server_rx)
+            try:
+                self._mailbox = self._app.open_mailbox(mailbox_id, self._side,
+                                                       server_rx)
+            except CrowdedError:
+                raise Error("crowded")
         if self._listening:
             self._mailbox.remove_listener(self)
             self._listening = False
@@ -281,6 +298,7 @@ class WebSocketRendezvous(websocket.WebSocketServerProtocol):
 
 class WebSocketRendezvousFactory(websocket.WebSocketServerFactory):
     protocol = WebSocketRendezvous
+
     def __init__(self, url, rendezvous):
         websocket.WebSocketServerFactory.__init__(self, url)
         self.setProtocolOptions(autoPingInterval=60, autoPingTimeout=600)
