@@ -3,15 +3,86 @@ import os, json, itertools, time
 import mock
 from twisted.trial import unittest
 from twisted.python import log
-from twisted.internet import reactor, defer
+from twisted.internet import reactor, defer, endpoints
 from twisted.internet.defer import inlineCallbacks, returnValue
 from autobahn.twisted import websocket
-from .. import __version__
 from .common import ServerBase
 from ..server import server, rendezvous
 from ..server.rendezvous import Usage, SidedMessage
 from ..server.database import get_db
 
+def easy_relay(
+        rendezvous_web_port=str("tcp:0"),
+        transit_port=str("tcp:0"),
+        advertise_version=None,
+        **kwargs
+):
+    return server.RelayServer(
+        rendezvous_web_port,
+        transit_port,
+        advertise_version,
+        **kwargs
+    )
+
+class RLimits(unittest.TestCase):
+    def test_rlimit(self):
+        def patch_s(name, *args, **kwargs):
+            return mock.patch("wormhole.server.server." + name, *args, **kwargs)
+        # We never start this, so the endpoints can be fake.
+        # serverFromString() requires bytes on py2 and str on py3, so this
+        # is easier than just passing "tcp:0"
+        ep = endpoints.TCP4ServerEndpoint(None, 0)
+        with patch_s("endpoints.serverFromString", return_value=ep):
+            s = server.RelayServer("fake", None, None)
+        fakelog = []
+        def checklog(*expected):
+            self.assertEqual(fakelog, list(expected))
+            fakelog[:] = []
+        NF = "NOFILE"
+        mock_NF = patch_s("RLIMIT_NOFILE", NF)
+
+        with patch_s("log.msg", fakelog.append):
+            with patch_s("getrlimit", None):
+                s.increase_rlimits()
+            checklog("unable to import 'resource', leaving rlimit alone")
+
+            with mock_NF:
+                with patch_s("getrlimit", return_value=(20000, 30000)) as gr:
+                    s.increase_rlimits()
+                    self.assertEqual(gr.mock_calls, [mock.call(NF)])
+                    checklog("RLIMIT_NOFILE.soft was 20000, leaving it alone")
+
+                with patch_s("getrlimit", return_value=(10, 30000)) as gr:
+                    with patch_s("setrlimit", side_effect=TypeError("other")):
+                        with patch_s("log.err") as err:
+                            s.increase_rlimits()
+                        self.assertEqual(err.mock_calls, [mock.call()])
+                        checklog("changing RLIMIT_NOFILE from (10,30000) to (30000,30000)",
+                                 "other error during setrlimit, leaving it alone")
+
+                    for maxlimit in [40000, 20000, 9000, 2000, 1000]:
+                        def setrlimit(which, newlimit):
+                            if newlimit[0] > maxlimit:
+                                raise ValueError("nope")
+                            return None
+                        calls = []
+                        expected = []
+                        for tries in [30000, 10000, 3200, 1024]:
+                            calls.append(mock.call(NF, (tries, 30000)))
+                            expected.append("changing RLIMIT_NOFILE from (10,30000) to (%d,30000)" % tries)
+                            if tries > maxlimit:
+                                expected.append("error during setrlimit: nope")
+                            else:
+                                expected.append("setrlimit successful")
+                                break
+                        else:
+                            expected.append("unable to change rlimit, leaving it alone")
+
+                        with patch_s("setrlimit", side_effect=setrlimit) as sr:
+                            s.increase_rlimits()
+                        self.assertEqual(sr.mock_calls, calls)
+                        checklog(*expected)
+
 class _Util:
     def _nameplate(self, app, name):
         np_row = app._db.execute("SELECT * FROM `nameplates`"
@@ -301,7 +372,7 @@ class Prune(unittest.TestCase):
 
     def test_update(self):
         db = get_db(":memory:")
-        rv = rendezvous.Rendezvous(db, None, None)
+        rv = rendezvous.Rendezvous(db, None, None, True)
         app = rv.get_app("appid")
         mbox_id = "mbox1"
         app.open_mailbox(mbox_id, "side1", 1)
@@ -315,7 +386,7 @@ class Prune(unittest.TestCase):
         self.assertEqual(self._get_mailbox_updated(app, mbox_id), 3)
 
     def test_apps(self):
-        rv = rendezvous.Rendezvous(get_db(":memory:"), None, None)
+        rv = rendezvous.Rendezvous(get_db(":memory:"), None, None, True)
         app = rv.get_app("appid")
         app.allocate_nameplate("side", 121)
         app.prune = mock.Mock()
@@ -324,7 +395,7 @@ class Prune(unittest.TestCase):
 
     def test_nameplates(self):
         db = get_db(":memory:")
-        rv = rendezvous.Rendezvous(db, None, 3600)
+        rv = rendezvous.Rendezvous(db, None, 3600, True)
 
         # timestamps <=50 are "old", >=51 are "new"
         #OLD = "old"; NEW = "new"
@@ -358,7 +429,7 @@ class Prune(unittest.TestCase):
 
     def test_mailboxes(self):
         db = get_db(":memory:")
-        rv = rendezvous.Rendezvous(db, None, 3600)
+        rv = rendezvous.Rendezvous(db, None, 3600, True)
 
         # timestamps <=50 are "old", >=51 are "new"
         #OLD = "old"; NEW = "new"
@@ -404,7 +475,7 @@ class Prune(unittest.TestCase):
         log.msg(desc)
 
         db = get_db(":memory:")
-        rv = rendezvous.Rendezvous(db, None, 3600)
+        rv = rendezvous.Rendezvous(db, None, 3600, True)
         APPID = "appid"
         app = rv.get_app(APPID)
 
@@ -606,7 +677,7 @@ class WSClientSync(unittest.TestCase):
 class WebSocketAPI(_Util, ServerBase, unittest.TestCase):
     def setUp(self):
         self._clients = []
-        return ServerBase.setUp(self)
+        self._setup_relay(None, advertise_version="advertised.version")
 
     def tearDown(self):
         for c in self._clients:
@@ -625,7 +696,7 @@ class WebSocketAPI(_Util, ServerBase, unittest.TestCase):
     def check_welcome(self, data):
         self.failUnlessIn("welcome", data)
         self.failUnlessEqual(data["welcome"],
-                             {"current_cli_version": __version__})
+                             {"current_cli_version": "advertised.version"})
 
     @inlineCallbacks
     def test_welcome(self):
@@ -893,6 +964,22 @@ class WebSocketAPI(_Util, ServerBase, unittest.TestCase):
         self.assertEqual(err["type"], "error")
         self.assertEqual(err["error"], "only one open per connection")
 
+    @inlineCallbacks
+    def test_open_crowded(self):
+        c1 = yield self.make_client()
+        yield c1.next_non_ack()
+        c1.send("bind", appid="appid", side="side")
+        app = self._rendezvous.get_app("appid")
+
+        mbid = app.claim_nameplate("np1", "side1", 0)
+        app.claim_nameplate("np1", "side2", 0)
+
+        # the third open will signal crowding
+        c1.send("open", mailbox=mbid)
+        err = yield c1.next_non_ack()
+        self.assertEqual(err["type"], "error")
+        self.assertEqual(err["error"], "crowded")
+
     @inlineCallbacks
     def test_add(self):
         c1 = yield self.make_client()
@@ -992,6 +1079,22 @@ class WebSocketAPI(_Util, ServerBase, unittest.TestCase):
         self.assertEqual(err["type"], "error")
         self.assertEqual(err["error"], "open and close must use same mailbox")
 
+    @inlineCallbacks
+    def test_close_crowded(self):
+        c1 = yield self.make_client()
+        yield c1.next_non_ack()
+        c1.send("bind", appid="appid", side="side")
+        app = self._rendezvous.get_app("appid")
+
+        mbid = app.claim_nameplate("np1", "side1", 0)
+        app.claim_nameplate("np1", "side2", 0)
+
+        # a close that allocates a third side will signal crowding
+        c1.send("close", mailbox=mbid)
+        err = yield c1.next_non_ack()
+        self.assertEqual(err["type"], "error")
+        self.assertEqual(err["error"], "crowded")
+
 
     @inlineCallbacks
     def test_disconnect(self):
@@ -1174,7 +1277,7 @@ class WebSocketAPI(_Util, ServerBase, unittest.TestCase):
 
 class Summary(unittest.TestCase):
     def test_mailbox(self):
-        app = rendezvous.AppNamespace(None, None, False, None)
+        app = rendezvous.AppNamespace(None, None, False, None, True)
         # starts at time 1, maybe gets second open at time 3, closes at 5
         def s(rows, pruned=False):
             return app._summarize_mailbox(rows, 5, pruned)
@@ -1217,7 +1320,7 @@ class Summary(unittest.TestCase):
         self.assertEqual(s(rows, pruned=True), Usage(1, 2, 4, "crowded"))
 
     def test_nameplate(self):
-        a = rendezvous.AppNamespace(None, None, False, None)
+        a = rendezvous.AppNamespace(None, None, False, None, True)
         # starts at time 1, maybe gets second open at time 3, closes at 5
         def s(rows, pruned=False):
             return a._summarize_nameplate_usage(rows, 5, pruned)
@@ -1233,10 +1336,21 @@ class Summary(unittest.TestCase):
         rows = [dict(added=1), dict(added=3), dict(added=4)]
         self.assertEqual(s(rows), Usage(1, 2, 4, "crowded"))
 
+    def test_nameplate_disallowed(self):
+        db = get_db(":memory:")
+        a = rendezvous.AppNamespace(db, None, False, "some_app_id", False)
+        a.allocate_nameplate("side1", "123")
+        self.assertEqual([], a.get_nameplate_ids())
+
+    def test_nameplate_allowed(self):
+        db = get_db(":memory:")
+        a = rendezvous.AppNamespace(db, None, False, "some_app_id", True)
+        np = a.allocate_nameplate("side1", "321")
+        self.assertEqual(set([np]), a.get_nameplate_ids())
 
     def test_blur(self):
         db = get_db(":memory:")
-        rv = rendezvous.Rendezvous(db, None, 3600)
+        rv = rendezvous.Rendezvous(db, None, 3600, True)
         APPID = "appid"
         app = rv.get_app(APPID)
         app.claim_nameplate("npid", "side1", 10) # start time is 10
@@ -1253,7 +1367,7 @@ class Summary(unittest.TestCase):
 
     def test_no_blur(self):
         db = get_db(":memory:")
-        rv = rendezvous.Rendezvous(db, None, None)
+        rv = rendezvous.Rendezvous(db, None, None, True)
         APPID = "appid"
         app = rv.get_app(APPID)
         app.claim_nameplate("npid", "side1", 10) # start time is 10
@@ -1271,7 +1385,7 @@ class Summary(unittest.TestCase):
 
 class DumpStats(unittest.TestCase):
     def test_nostats(self):
-        rs = server.RelayServer(str("tcp:0"), str("tcp:0"), None)
+        rs = easy_relay()
         # with no ._stats_file, this should do nothing
         rs.dump_stats(1, 1)
 
@@ -1279,8 +1393,7 @@ class DumpStats(unittest.TestCase):
         basedir = self.mktemp()
         os.mkdir(basedir)
         fn = os.path.join(basedir, "stats.json")
-        rs = server.RelayServer(str("tcp:0"), str("tcp:0"), None,
-                                stats_file=fn)
+        rs = easy_relay(stats_file=fn)
         now = 1234
         validity = 500
         rs.dump_stats(now, validity)
@@ -1292,3 +1405,31 @@ class DumpStats(unittest.TestCase):
         self.assertEqual(data["rendezvous"]["all_time"]["mailboxes_total"], 0)
         self.assertEqual(data["transit"]["all_time"]["total"], 0)
 
+
+class Startup(unittest.TestCase):
+
+    @mock.patch('wormhole.server.server.log')
+    def test_empty(self, fake_log):
+        rs = easy_relay(allow_list=False)
+        rs.startService()
+        try:
+            logs = '\n'.join([call[1][0] for call in fake_log.mock_calls])
+            self.assertTrue(
+                'listing of allocated nameplates disallowed' in logs
+            )
+        finally:
+            rs.stopService()
+
+
+class WebSocketProtocolOptions(unittest.TestCase):
+    @mock.patch('wormhole.server.server.WebSocketRendezvousFactory')
+    def test_set(self, fake_factory):
+        easy_relay(
+            websocket_protocol_options=[
+                ("foo", "bar"),
+            ]
+        )
+        self.assertEqual(
+            mock.call().setProtocolOptions(foo="bar"),
+            fake_factory.mock_calls[1],
+        )
