@@ -138,7 +138,7 @@ def parse_hint_argv(hint, stderr=sys.stderr):
                 return None
     return DirectTCPV1Hint(hint_host, hint_port, priority)
 
-TIMEOUT=15
+TIMEOUT = 60 # seconds
 
 @implementer(interfaces.IProducer, interfaces.IConsumer)
 class Connection(protocol.Protocol, policies.TimeoutMixin):
@@ -243,8 +243,11 @@ class Connection(protocol.Protocol, policies.TimeoutMixin):
             raise BadHandshake("abandoned")
         if self.state == "records":
             return self.dataReceivedRECORDS()
+        if self.state == "hung up":
+            return
         if isinstance(self.state, Exception): # for tests
             raise self.state
+        raise ValueError("internal error: unknown state %s" % (self.state,))
 
     def _negotiationSuccessful(self):
         self.state = "records"
@@ -374,7 +377,8 @@ class Connection(protocol.Protocol, policies.TimeoutMixin):
         least that number of bytes have been written. This function will then
         return a Deferred (that fires with the number of bytes actually
         received). If the connection is lost while this Deferred is
-        outstanding, it will errback.
+        outstanding, it will errback. If 'expected' is 0, the Deferred will
+        fire right away.
 
         If 'expected' is None, then this function returns None instead of a
         Deferred, and you must call disconnectConsumer() when you are done."""
@@ -402,6 +406,9 @@ class Connection(protocol.Protocol, policies.TimeoutMixin):
         if expected is not None:
             d = defer.Deferred()
         self._consumer_deferred = d
+        if expected == 0:
+            # write empty record to kick consumer into shutdown
+            self._writeToConsumer(b"")
         # drain any pending records
         while self._consumer and self._inbound_records:
             r = self._inbound_records.popleft()
@@ -428,6 +435,7 @@ class Connection(protocol.Protocol, policies.TimeoutMixin):
     # optional callable which will be called on each write (with the number
     # of bytes written). Returns a Deferred that fires (with the number of
     # bytes written) when the count is reached or the RecordPipe is closed.
+
     def writeToFile(self, f, expected, progress=None, hasher=None):
         fc = FileConsumer(f, progress, hasher)
         return self.connectConsumer(fc, expected)
@@ -581,7 +589,7 @@ class Common:
     RELAY_DELAY = 2.0
     TRANSIT_KEY_LENGTH = SecretBox.KEY_SIZE
 
-    def __init__(self, transit_relay, no_listen=False, tor_manager=None,
+    def __init__(self, transit_relay, no_listen=False, tor=None,
                  reactor=reactor, timing=None):
         self._side = bytes_to_hexstr(os.urandom(8)) # unicode
         if transit_relay:
@@ -595,7 +603,7 @@ class Common:
             self._transit_relays = []
         self._their_direct_hints = [] # hintobjs
         self._our_relay_hints = set(self._transit_relays)
-        self._tor_manager = tor_manager
+        self._tor = tor
         self._transit_key = None
         self._no_listen = no_listen
         self._waiting_for_transit_key = []
@@ -606,7 +614,7 @@ class Common:
         self._timing.add("transit")
 
     def _build_listener(self):
-        if self._no_listen or self._tor_manager:
+        if self._no_listen or self._tor:
             return ([], None)
         portnum = allocate_tcp_port()
         addresses = ipaddrs.find_addresses()
@@ -696,7 +704,8 @@ class Common:
                and isinstance(hint[u"hostname"], type(u""))):
             log.msg("invalid hostname in hint: %r" % (hint,))
             return None
-        if not(u"port" in hint and isinstance(hint[u"port"], int)):
+        if not(u"port" in hint
+               and isinstance(hint[u"port"], six.integer_types)):
             log.msg("invalid port in hint: %r" % (hint,))
             return None
         priority = hint.get(u"priority", 0.0)
@@ -811,6 +820,8 @@ class Common:
             if not ep:
                 continue
             description = "->%s" % describe_hint_obj(hint_obj)
+            if self._tor:
+                description = "tor" + description
             d = self._start_connector(ep, description)
             contenders.append(d)
             relay_delay = self.RELAY_DELAY
@@ -836,6 +847,8 @@ class Common:
                 if not ep:
                     continue
                 description = "->relay:%s" % describe_hint_obj(hint_obj)
+                if self._tor:
+                    description = "tor" + description
                 d = task.deferLater(self._reactor, relay_delay,
                                     self._start_connector, ep, description,
                                     is_relay=True)
@@ -874,12 +887,14 @@ class Common:
         return d
 
     def _endpoint_from_hint_obj(self, hint):
-        if self._tor_manager:
+        if self._tor:
             if isinstance(hint, (DirectTCPV1Hint, TorTCPV1Hint)):
-                # our TorManager will return None for non-public IPv4
+                # this Tor object will throw ValueError for non-public IPv4
                 # addresses and any IPv6 address
-                return self._tor_manager.get_endpoint_for(hint.hostname,
-                                                          hint.port)
+                try:
+                    return self._tor.stream_via(hint.hostname, hint.port)
+                except ValueError:
+                    return None
             return None
         if isinstance(hint, DirectTCPV1Hint):
             return endpoints.HostnameEndpoint(self._reactor,
